#========================================================================
# NAM: AnkiConf
#
# ROL: Very simple package used to manage configuration files similar
#      to those used in Windows <=3.11 (.INI files).
#
#                            -oOo-
#
#      This source can be distributed without any permission.
#      If you use it, tell me about it !
#
#      Ce source est distribuable sans autorisation.
#      S'il vous a servi, dites le moi !
#
# AUT: AnkiF (ankif@mailoman.com)
#
# $Id: AnkiConf.tcl,v 1.2.1.2 2000/05/12 00:55:22 fran Exp fran $
#
#========================================================================

package provide AnkiConf 0.3

package require msgcat

namespace eval AnkiConf {
variable ListeSect ""

namespace export *
}

#---------------------------------------------------
# Variables de l'application
#---------------------------------------------------


#***********************************************************************
#                     FONCTIONS GENERIQUES I/O                         *
#***********************************************************************

#========================================================================
# FCT: ReadFile
# ROL: Lecture d'un fichier de configuration.
# ARG: fichier    (E):
# ARG: buffer     (S):
#
#========================================================================
proc AnkiConf::ReadFile { fichier } {
variable ListeSect

   set result [catch "open $fichier r" fd]
   if { $result != 0 } return ;

  
   set count [gets $fd ligne]
   while {$count >= 0} {
      switch [string index $ligne 0] {
         "\[" {
               # Cas d'un dbut de section: ajout dans la liste des sections
               scan $ligne {[%[^]]} nomsect
               lappend ListeSect $nomsect
              }
         "\n" {}
         " "  {}
         "\t" {}
         "#"  {
               # Cas d'un commentaire, on mmorise pour plus tard
               set comment [string trim [string range $ligne 1 end]]
              }
         default {
               # Cas d'une ligne dfinissant un mot-cl
               scan $ligne {%[^=]} motcle
               if [info exists motcle] {
                  scan $ligne {%*s %s %[^]]} typdon valdon

                  switch $typdon {
                      "string" { 
                          set longueur [string length $valdon]
                          variable ${nomsect}
                          set ${nomsect}($motcle,val) \
					   "[string range $valdon 1 [expr $longueur-2]]"
                      }
                      default  { 
                          variable ${nomsect}
                          set ${nomsect}($motcle,val) "$valdon"
                      }
                  }
                  set ${nomsect}($motcle,type) $typdon
                  if [info exists comment] {
                     set ${nomsect}($motcle,com) $comment
                     unset comment
                  }
               }
           }
        }

       set count [gets $fd ligne]

   }

   close $fd
}



#========================================================================
# FCT: WriteFile
# ROL: Ecriture d'un fichier de configuration.
# ARG: fichier    (E):
# ARG: buffer     (S):
#
#========================================================================
proc ::AnkiConf::WriteFile { fichier } {
variable ListeSect

   set fd [open $fichier "w"]
   if { $fd != "" } \
     {
      foreach section $ListeSect \
        {
         puts $fd \[$section\]
         variable $section

         foreach indice [array names $section *,val] \
           {
            set motcle [string range $indice 0 [expr [string length $indice]-5]]
            set typdon [set ${section}($motcle,type)]
            set valdon [set ${section}($motcle,val)]

            switch $typdon {
                "string" { set valdon \"$valdon\" }
                default  {  }
               }
            if [info exists ${section}($motcle,com)] {
                puts $fd "# [set ${section}($motcle,com)]"
               }
            puts $fd "$motcle= $typdon $valdon"
           }
        }
      close $fd
     }
}



#========================================================================
# FCT: GetVar
# ROL: Lecture d'un paramtre dans le fichier de configuration.
# ARG: nant
#
#========================================================================
proc ::AnkiConf::GetVar { section motcle {valdef {}}} {
variable ListeSect

      if { [lsearch -exact $ListeSect $section] >= 0 } {
         variable $section
         if [info exists ${section}($motcle,val)] {
            return [set ${section}($motcle,val)]
           } 
        }
 return "$valdef"
}




#========================================================================
# FCT: SetVar
# ROL: Affectation d'une variable
# ARG: nant
#
#========================================================================
proc ::AnkiConf::SetVar { section motcle valeur } {
variable ListeSect

      if { [lsearch -exact $ListeSect $section] >= 0 } {
          variable $section
          if [info exists ${section}($motcle,val)] {
             set ${section}($motcle,val) $valeur
            }
        } 
}


#========================================================================
# FCT: CreateVar
# ROL: Cration d'une variable
# ARG: section     (I):
#      motcle      (I): Keyword
#      valeur      (I): Value
#      type        (I):
#
#========================================================================
proc ::AnkiConf::CreateVar { section motcle valeur type} {
variable ListeSect

      if { [lsearch -exact $ListeSect $section] < 0 } {
         lappend ListeSect $section
        } 
      variable $section
      set ${section}($motcle,val) $valeur
      set ${section}($motcle,type) $type
}



#========================================================================
# FCT: CreateString
# ROL: Cration d'une variable chaine
# ARG: nant
#
#========================================================================
proc ::AnkiConf::CreateString { section motcle valeur } {

      CreateVar $section $motcle $valeur "string"
}


#========================================================================
# FCT: CreateInt
# ROL: Cration d'une variable entire
# ARG: nant
#
#========================================================================
proc ::AnkiConf::CreateInt { section motcle valeur } {

      CreateVar $section $motcle $valeur "int"
}



#========================================================================
# FCT: AppliqueModifs
# ROL: Met  jour les variables
# ARG: section
#
#========================================================================
proc ::AnkiConf::AppliqueModifs { panel section } {
variable $section

    foreach indice [array names $section *,val] {
      
       set motcle [string range $indice 0 [expr [string length $indice]-5]]
       set typdon [set ${section}($motcle,type)]

       # extraction ventuelle d'arguments du type
       set index [string first "(" $typdon ]
       if { $index >= 0 } {
          set args [split [string range $typdon $index end] "(,)"]
          incr index -1
          set typdon [string range $typdon 0 $index]
       }

       # Frame qui regroupe le label et l'item
       set group $panel.[string tolower $motcle]

       # Valeur = dpend du type
       switch $typdon {
          "list"   { set ${section}($motcle,val) [${group}_v cget -text] }
          "bool"   { global AnkiConf:${group}_v ; set ${section}($motcle,val) [ set AnkiConf:${group}_v] }
          "string"  -
          "int"    { set ${section}($motcle,val) [${group}_v get] }
          default  { puts "Unknow type : $typdon ; can't update internal state" }
       }
   }
}


#========================================================================
# FCT: EditDialog
# ROL: Build a dialog window allowing to edit all the variables in
#      a section
# ARG: w         (I): Parent window
#      section   (I): section name
#      titre     (I): Window title
#
#========================================================================
proc ::AnkiConf::EditDialog { w section {titre {No title}} } {
variable ListeSect

    if { [lsearch -exact $ListeSect $section] >= 0 } {
       variable $section

       #  The toplevel will be named after the section name,
       # allowing to have several configuration windows simultaneously
       #
       set section_as_a_word [string tolower [join [split $section] _]]

       if { "$w" == "." } {
          set nom .aked_${section_as_a_word}
       } else {
          set nom $w.aked_${section_as_a_word}
       }
       if [ winfo exists $nom ] return ;

       set ed [toplevel ${nom}]
       wm title $ed $titre


       #  Cration panel du haut (grille)
       set haut [frame $ed.p]
       grid rowconfig   $haut 0 -weight 1 -minsize 20
       grid columnconfig $haut 0 -weight 2 -minsize 20
       grid columnconfig $haut 1 -weight 1 -minsize 20

       set ligne 0
       foreach indice [array names $section *,val] \
         {
          set motcle [string range $indice 0 [expr [string length $indice]-5]]
          set valdon [set ${section}($motcle,val)]
          set typdon [set ${section}($motcle,type)]

          # extraction ventuelle d'arguments du type
          set index [string first "(" $typdon ]

          if { $index >= 0 } {
             set args [split [string range $typdon $index end] "(,)"]
             incr index -1
             set typdon [string range $typdon 0 $index]
          }


          # Label = mot-cl ventuellement traduit dans le namespace
          # de l'appelant

          set group $haut.[string tolower $motcle]
          label ${group}_l -text [uplevel "::msgcat::mc $motcle"]
          grid ${group}_l -padx 1 -in $haut -pady 1 \
                -column 0 -row $ligne -rowspan 1 -columnspan 1 -sticky nsw



          # Valeur = dpend du type

          switch $typdon {
              "string" { entry ${group}_v 
                         ${group}_v insert end $valdon }

              "int"    { entry ${group}_v 
                         ${group}_v insert end $valdon }

              "bool"   { frame ${group}_v
                         global AnkiConf:${group}_v
                         set AnkiConf:${group}_v $valdon
                         radiobutton ${group}_v.1 -text [::msgcat::mc "yes"] \
                                                  -value "yes" -variable AnkiConf:${group}_v
                         radiobutton ${group}_v.2 -text [::msgcat::mc "no"] \
                                                  -value "no" -variable AnkiConf:${group}_v
                         pack ${group}_v.1 ${group}_v.2 -side left
                       }

              "list"   { menubutton ${group}_v -text $valdon -menu ${group}_v.m \
                                             -indicator 1 -relief groove
                         menu ${group}_v.m -tearoff 0
                         foreach item $args {
                            if { $item != "" } { 
                               ${group}_v.m add command -label $item \
                                    -command "set valdon $item ; ${group}_v config -text $item"
                              }
                           }
                       }
              default  {  }
             }

          # 
          grid ${group}_v -padx 1 -in $haut -pady 1 \
                -column 1 -row $ligne -rowspan 1 -columnspan 1 -sticky nsw
          incr ligne
         }
     pack $haut -side top

     # Panel des boutons
     #
     frame $ed.box
     button $ed.box.ok     -text Ok -width 6  -default active \
                           -command "::AnkiConf::AppliqueModifs $haut $section ; destroy $ed"
     button $ed.box.cancel -text Cancel -width 6 \
                           -command "destroy $ed"
     pack $ed.box.ok $ed.box.cancel -padx 10 -side left
     pack $ed.box -side bottom -expand no -pady 4

     bind $ed <Return> "$ed.box.ok invoke"
     bind $ed <Escape> "$ed.box.cancel invoke"
    }
}

